@@ -1,6 +1,7 @@
 # Tests of Python-specific parts of the xapian bindings.
 #
 # Copyright (C) 2007 Lemur Consulting Ltd
+# Copyright (C) 2009 Olly Betts
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -793,6 +794,56 @@ def test_synonyms_iter():
     del dbr
     shutil.rmtree(dbpath)
 
+def test_metadata_keys_iter():
+    """Test iterators over list of metadata keys in a database.
+
+    """
+    dbpath = 'flinttest_metadata_iter'
+    db = xapian.WritableDatabase(dbpath, xapian.DB_CREATE_OR_OVERWRITE)
+
+    db.set_metadata('author', 'richard')
+    db.set_metadata('item1', 'hello')
+    db.set_metadata('item1', 'hi')
+    db.set_metadata('item2', 'howdy')
+    db.set_metadata('item3', '')
+    db.set_metadata('item4', 'goodbye')
+    db.set_metadata('item4', '')
+    db.set_metadata('type', 'greeting')
+
+    expect([item for item in db.metadata_keys()],
+           ['author', 'item1', 'item2', 'type'])
+    expect([item for item in db.metadata_keys('foo')], [])
+    expect([item for item in db.metadata_keys('item')], ['item1', 'item2'])
+    expect([item for item in db.metadata_keys('it')], ['item1', 'item2'])
+    expect([item for item in db.metadata_keys('type')], ['type'])
+
+    dbr=xapian.Database(dbpath)
+    expect([item for item in dbr.metadata_keys()], [])
+    expect([item for item in dbr.metadata_keys('foo')], [])
+    expect([item for item in dbr.metadata_keys('item')], [])
+    expect([item for item in dbr.metadata_keys('it')], [])
+    expect([item for item in dbr.metadata_keys('type')], [])
+
+    db.flush()
+    expect([item for item in db.metadata_keys()],
+           ['author', 'item1', 'item2', 'type'])
+    expect([item for item in db.metadata_keys('foo')], [])
+    expect([item for item in db.metadata_keys('item')], ['item1', 'item2'])
+    expect([item for item in db.metadata_keys('it')], ['item1', 'item2'])
+    expect([item for item in db.metadata_keys('type')], ['type'])
+
+    dbr=xapian.Database(dbpath)
+    expect([item for item in dbr.metadata_keys()],
+           ['author', 'item1', 'item2', 'type'])
+    expect([item for item in dbr.metadata_keys('foo')], [])
+    expect([item for item in dbr.metadata_keys('item')], ['item1', 'item2'])
+    expect([item for item in dbr.metadata_keys('it')], ['item1', 'item2'])
+    expect([item for item in dbr.metadata_keys('type')], ['type'])
+
+    del db
+    del dbr
+    shutil.rmtree(dbpath)
+
 def test_spell():
     """Test basic spelling correction features.
 
@@ -837,6 +888,28 @@ def test_queryparser_custom_vrp():
     expect(str(query),
            'Xapian::Query(VALUE_RANGE 7 A5 B8)')
 
+def test_queryparser_custom_vrp_deallocation():
+    """Test that QueryParser doesn't delete ValueRangeProcessors too soon.
+
+    """
+    class MyVRP(xapian.ValueRangeProcessor):
+        def __init__(self):
+            xapian.ValueRangeProcessor.__init__(self)
+
+        def __call__(self, begin, end):
+            return (7, "A"+begin, "B"+end)
+
+    def make_parser():
+        queryparser = xapian.QueryParser()
+        myvrp = MyVRP()
+        queryparser.add_valuerangeprocessor(myvrp)
+        return queryparser
+
+    queryparser = make_parser()
+    query = queryparser.parse_query('5..8')
+
+    expect(str(query),
+           'Xapian::Query(VALUE_RANGE 7 A5 B8)')
 
 def test_scale_weight():
     """Test query OP_SCALE_WEIGHT feature.
@@ -858,8 +931,8 @@ def test_scale_weight():
             expected = [(0, item.docid) for item in mset1]
             expected.sort()
         else:
-            expected = [(item.weight * mult, item.docid) for item in mset1]
-        expect([(item.weight, item.docid) for item in mset2], expected)
+            expected = [(int(item.weight * mult * 1000000), item.docid) for item in mset1]
+        expect([(int(item.weight * 1000000), item.docid) for item in mset2], expected)
 
     context("checking queries with OP_SCALE_WEIGHT with a multipler of -1")
     query1 = xapian.Query("it")
@@ -916,12 +989,120 @@ def test_weight_normalise():
             expect(item.weight > 0, True)
             expect(item.weight <= 1, True)
 
+    del db
+
+def test_director_exception():
+    """Test handling of an exception raised in a director.
+
+    """
+    dbpath = 'db_test_value_stats'
+    db = setup_database()
+    query = xapian.Query('it')
+    enq = xapian.Enquire(db)
+    enq.set_query(query)
+    class TestException(Exception):
+        def __init__(self, a, b):
+            Exception.__init__(self, a + b)
+
+    rset = xapian.RSet()
+    rset.add_document(1)
+    class EDecider(xapian.ExpandDecider):
+        def __call__(self, term):
+            raise TestException("foo", "bar")
+    edecider = EDecider()
+    expect_exception(TestException, "foobar", edecider, "foo")
+    expect_exception(TestException, "foobar", enq.get_eset, 10, rset, edecider)
+
+    class MDecider(xapian.MatchDecider):
+        def __call__(self, doc):
+            raise TestException("foo", "bar")
+    mdecider = MDecider()
+    expect_exception(TestException, "foobar", mdecider, xapian.Document())
+    expect_exception(TestException, "foobar", enq.get_mset, 0, 10, None, mdecider)
 
 # The legacy sequence API is only supported for Python >= 2.3 so don't try
 # testing it for Python 2.2.
 vinfo = sys.version_info    
 test_legacy_sequence_api = vinfo[0] > 2 or (vinfo[0] == 2 and vinfo[1] >= 3)
 
+def test_preserve_query_parser_stopper():
+    """Test preservation of stopper set on query parser.
+
+    """
+    def make_qp():
+        queryparser = xapian.QueryParser()
+        stopper = xapian.SimpleStopper()
+        stopper.add('to')
+        stopper.add('not')
+        queryparser.set_stopper(stopper)
+        del stopper
+        return queryparser
+    queryparser = make_qp()
+    query = queryparser.parse_query('to be')
+    expect([term for term in queryparser.stoplist()], ['to']) 
+
+def test_preserve_term_generator_stopper():
+    """Test preservation of stopper set on term generator.
+
+    """
+    def make_tg():
+        termgen = xapian.TermGenerator()
+        termgen.set_stemmer(xapian.Stem('en'))
+        stopper = xapian.SimpleStopper()
+        stopper.add('to')
+        stopper.add('not')
+        termgen.set_stopper(stopper)
+        del stopper
+        return termgen
+    termgen = make_tg()
+
+    termgen.index_text('to be')
+    doc = termgen.get_document()
+    terms = [term.term for term in doc.termlist()]
+    terms.sort()
+    expect(terms, ['Zbe', 'be', 'to']) 
+
+def test_preserve_enquire_sorter():
+    """Test preservation of sorter set on enquire.
+
+    """
+    db = xapian.inmemory_open()
+    doc = xapian.Document()
+    doc.add_term('foo')
+    doc.add_value(1, '1')
+    db.add_document(doc)
+    db.add_document(doc)
+
+    def make_enq1(db):
+        enq = xapian.Enquire(db)
+        sorter = xapian.MultiValueSorter()
+        enq.set_sort_by_key(sorter, ascending = True)
+        del sorter
+        return enq
+    enq = make_enq1(db)
+    enq.set_query(xapian.Query('foo'))
+    enq.get_mset(0, 10)
+
+    def make_enq2(db):
+        enq = xapian.Enquire(db)
+        sorter = xapian.MultiValueSorter()
+        enq.set_sort_by_key_then_relevance(sorter, reverse = True)
+        del sorter
+        return enq
+    enq = make_enq2(db)
+    enq.set_query(xapian.Query('foo'))
+    enq.get_mset(0, 10)
+
+    def make_enq3(db):
+        enq = xapian.Enquire(db)
+        sorter = xapian.MultiValueSorter()
+        enq.set_sort_by_relevance_then_key(sorter, True)
+        del sorter
+        return enq
+    enq = make_enq3(db)
+    enq.set_query(xapian.Query('foo'))
+    enq.get_mset(0, 10)
+
 # Run all tests (ie, callables with names starting "test_").
 if not runtests(globals(), sys.argv[1:]):
     sys.exit(1)
